@@ -1,8 +1,4 @@
-
-
-
-
-use super::{network_address::NetworkAddress};
+use super::network_address::NetworkAddress;
 
 #[derive(Clone, Copy)]
 #[repr(C)]
@@ -11,7 +7,7 @@ pub struct Connection {
     pub identity: Identity,
     pub tachyon_id: u16,
     pub received_at: u64,
-    pub since_last_received: u64
+    pub since_last_received: u64,
 }
 
 impl Connection {
@@ -21,12 +17,10 @@ impl Connection {
             address: address,
             tachyon_id,
             received_at: 0,
-            since_last_received: 0
+            since_last_received: 0,
         };
         return conn;
     }
-
-  
 }
 
 #[derive(Clone, Copy)]
@@ -35,11 +29,10 @@ impl Connection {
 pub struct Identity {
     pub id: u32,
     pub session_id: u32,
-    pub linked: u32
+    pub linked: u32,
 }
 
 impl Identity {
-    
     pub fn is_valid(&self) -> bool {
         return self.id > 0 && self.session_id > 0;
     }
@@ -52,4 +45,3 @@ impl Identity {
         self.linked = linked;
     }
 }
-
