@@ -1,22 +1,25 @@
-use std::{net::{SocketAddr, Ipv4Addr, IpAddr}, hash::{Hash, Hasher}};
+use std::{
+    hash::{Hash, Hasher},
+    net::{IpAddr, Ipv4Addr, SocketAddr},
+};
 
-
-#[derive(Eq)]
-#[derive(Default)]
-#[derive(Clone, Copy)]
+#[derive(Eq, Default, Clone, Copy)]
 #[repr(C)]
 pub struct NetworkAddress {
     pub a: u16,
     pub b: u16,
     pub c: u16,
     pub d: u16,
-    pub port: u32
+    pub port: u32,
 }
 
 impl std::fmt::Display for NetworkAddress {
     fn fmt(&self, f: &mut std::fmt::Formatter) -> std::fmt::Result {
-        write!(f,"{0}.{1}.{2}.{3}:{4}\n",
-         self.a, self.b, self.c, self.d, self.port)
+        write!(
+            f,
+            "{0}.{1}.{2}.{3}:{4}\n",
+            self.a, self.b, self.c, self.d, self.port
+        )
     }
 }
 
@@ -30,12 +33,15 @@ impl Hash for NetworkAddress {
 
 impl PartialEq for NetworkAddress {
     fn eq(&self, other: &Self) -> bool {
-        self.a == other.a && self.b == other.b && self.c == other.c && self.d == other.d && self.port == other.port
+        self.a == other.a
+            && self.b == other.b
+            && self.c == other.c
+            && self.d == other.d
+            && self.port == other.port
     }
 }
 
 impl NetworkAddress {
-    
     pub fn test_address() -> Self {
         let address = NetworkAddress {
             a: 127,
@@ -70,15 +76,14 @@ impl NetworkAddress {
     }
 
     pub fn from_socket_addr(address: SocketAddr) -> NetworkAddress {
-    
         if let IpAddr::V4(ipv4) = address.ip() {
             let parts = ipv4.octets();
             let result = NetworkAddress {
                 a: parts[0] as u16,
-                b:  parts[1] as u16,
+                b: parts[1] as u16,
                 c: parts[2] as u16,
                 d: parts[3] as u16,
-                port: address.port() as u32
+                port: address.port() as u32,
             };
             return result;
         } else {
@@ -86,8 +91,7 @@ impl NetworkAddress {
         }
     }
 
-    pub fn to_socket_addr(&self) -> SocketAddr
-    {
+    pub fn to_socket_addr(&self) -> SocketAddr {
         let ip = Ipv4Addr::new(self.a as u8, self.b as u8, self.c as u8, self.d as u8);
         return SocketAddr::new(IpAddr::V4(ip), self.port as u16);
     }
